#!/usr/bin/perl

# Author : Damien KROTKINE (damien@tuxfamily.org)
#
# Contributors : 
# Charles LONGEAU (chl@tuxfamily.org)
# Arnaud DESMONS (logarno@libconf.net)
#
# Copyright (C) 2002 Damien KROTKINE (damien@tuxfamily.org)
# Copyright (C) 2002 Charles LONGEAU (chl@tuxfamily.org)
# Copyright (C) 2004 Arnaud DESMONS (logarno@libconf.net)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package Libconf;

=head1 NAME

  Libconf - Set of abstraction layers to the linux/unix configuration

=head1 SYNOPSIS

=over

=item B<High level>

  use Libconf::Glueconf::Networking::Resolv;
  my $resolv = new Libconf::Glueconf::Networking::Resolv('/etc/resolv.conf');
  push(@{$resolv->{nameserver}}, '192.168.76.7');
  $resolv->{search}->[1] = 'foo';
  $resolv->writeConf('/etc/resolv_modified');

=item B<Low level> (don't use this unless you know what you're doing)

  $struct = new Libconf("smb.conf", 'samba', '');
  $struct->editAtom(-1, {key => 'writable', value => 'TEST', sections => [{name => 'homes'}] });
  $struct->appendAtom({key => 'K1', value => 'hello', type => 'KEY_VALUE' });
  $struct->writeConf("smb.conf");

=back

=head1 DESCRIPTION

=over

=item B<High level>

See L<Libconf::Glueconf> for more information on this layer

This high level layer provides a hierarchical structure representing the
configuration file informations. It's very simple to use, and it uses a
"semantic" approach to structure the informations. You should use this layer.

=item B<Low level> (don't use this unless you know what you're doing)

The low level layer is used to parse one configuration file. It gives in return
a structure that is simple, but not as easy to manipulate as the one
Libconf::Glueconf can provide. Libconf::Glueconf actually uses this structure
to map it onto perl hashes (using tie). You can modify the datas of the
structure, and write them back. The configuration informations are stored in a
sequential manner. The power of the low level layer, is that its structure is
unique for any configuration file type.

See L<Libconf::Templates> for more information on this layer

=cut

use strict;
use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK $VERSION %templates
 $Indentspaces
 $Templatesdirectory
 $Trimspaces
 $Strict
 );

@ISA = qw(Exporter);
require Exporter;
%EXPORT_TAGS = (
    functions => [ qw() ],
    settings => [ qw() ],
    debug => [ qw (debug) ],
    helpers => [ qw(cat_ output deepcopy member to_bool bool2truefalse bool2yesno text2bool map_index if_ fold_left compare_depth deepcompare_ put_in_hash add2hash add2hash_ is_installed can_XML can_XML_Twig can_DumpXML) ],
);
$EXPORT_TAGS{all} = [ map { @$_ } values %EXPORT_TAGS ];
@EXPORT_OK = map { @$_ } values %EXPORT_TAGS;

$VERSION='0.42.00';
$Indentspaces = '    ';
$Trimspaces = 1;
$Strict = 1;

sub getTimestamp {
    my ($filename) = @_;
    my @stat;
    @stat = stat($filename);
    $stat[9];
}


sub compare_depth {
    my ($ref1, $ref2) = @_;
    my @sections1 = @$ref1;
    my @sections2 = @$ref2;
    @sections1 == @sections2 or return 0;
    foreach my $i (0..@sections1-1) {
        $sections2[$i]->{name} eq $sections1[$i]->{name} or return 0;
    }
    1;
}

sub debug {
    $ENV{DEBUG} or return;
    my $i = 0;
    while (caller($i)) { $i++ }
    my ($_package, $_filename, $_line, $subroutine, $_hasargs, $_wantarray, $_evaltext, $_is_require, $_hints, $_bitmask) = caller(2);
    print '   ' x ($i-3);
    print "\033[35m";
    print $subroutine, "\t\033[36m", @_;
    print "\033[0m\n";
}

# Helper functions

#           open(my $fh,'<:encoding(Big5)', 'anything');

sub cat_ { my ($f, $e) = @_;
           local *F;
           open F, defined $e ? ("<:encoding($e)", $f) : $f or die "reading file $f failed: $!\n";
           my @l = <F>;
           wantarray() ? @l : join '', @l
       }
sub output { my $f = shift; local *F; open F, ">$f" or die "output in file $f failed: $!\n"; print F foreach @_; 1 }
sub member { my $e = shift; foreach (@_) { $e eq $_ and return 1 } 0 }
sub to_bool { $_[0] ? 1 : 0 }
sub bool2truefalse { $_[0] ? "true" : "false" }
sub bool2yesno { $_[0] ? "yes" : "no" }
sub text2bool { my $t = lc($_[0]); $t eq "true" || $t eq "yes" ? 1 : 0 }
sub put_in_hash { my ($a, $b) = @_; while (my ($k, $v) = each %{$b || {}}) { $a->{$k} = $v } $a }
sub add2hash    { my ($a, $b) = @_; while (my ($k, $v) = each %{$b || {}}) { $a->{$k} ||= $v } $a }
sub add2hash_   { my ($a, $b) = @_; while (my ($k, $v) = each %{$b || {}}) { exists $a->{$k} or $a->{$k} = $v } $a }
sub deepcopy {
    my $s = shift;
    ref($s) eq 'ARRAY' and return [ map deepcopy($_), @$s ];
    ref($s) eq 'HASH' and return { map { $_ => deepcopy($s->{$_}) } keys %$s };
    $s;
}
sub is_installed { (my $mod = "$_[0].pm" ) =~ s!::!/!g; -e "$_/$mod" && return 1 for @INC }

sub can_XML { is_installed('XML::LibXML') }
sub can_XML_Twig { is_installed('XML::Twig') }
sub can_DumpXML { 
    #is_installed('Data::DumpXML');
    is_installed('XML::Dumper');
}

sub deepcompare_ {
    my ($search_struct, $struct) = @_;
    if (ref($search_struct) eq 'HASH') {
        ref($struct) eq 'HASH' or return 0;
#        scalar(keys %$search_struct) == scalar(keys %$struct) or return 0;
        foreach (keys %$search_struct) {
            deepcompare_($search_struct->{$_}, $struct->{$_}) or return 0;
        }
        return 1;
    }
    if (ref($search_struct) eq 'ARRAY') {
        ref($struct) eq 'ARRAY' or return 0;
        scalar(@$search_struct) == scalar(@$struct) or return 0;
        foreach (0..$#{$search_struct}) {
            deepcompare_($search_struct->[$_], $struct->[$_]) or return 0;
        } 
        return 1;
    }
    $search_struct eq $struct;
}
sub map_index(&@) {
    my $f = shift;
    my @v; local $::i = 0;
    map { @v = &$f($::i); $::i++; @v } @_;
}
sub if_($@) { 
    my $b = shift;
    $b or return ();
    wantarray() || @_ <= 1 or die("if_ called in scalar context with more than one argument " . join(":", caller()));
    wantarray() ? @_ : $_[0];
}       
sub fold_left(&@) {
    my ($f, $initial, @l) = @_;
    local ($::a, $::b);
    $::a = $initial;
    foreach (@l) { $::b = $_; $::a = &$f() }
    $::a
}       

sub warn_old_api {
    warn("   *** Warning ***\n");
    warn("   * you are using the old template system\n");
    warn("   * This will be unsupported soon\n");
    warn("   * you should switch to Template API\n");
    die "OLD API";
}

=back

=head1 SEE ALSO

L<Libconf::Glueconf>, L<Libconf::Templates>

=cut

1;
